@@ -1,11 +1,10 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-import cluttergtk # must be the first to be imported
-import clutter
 import gobject
-import gtk
+
+import champlaingtk # must be the first to be imported
 import champlain
-import champlaingtk
+import gtk
 
 from markers import create_marker_layer
 
@@ -18,8 +17,17 @@ class LauncherGTK:
 
         vbox = gtk.VBox(False, 12)
 
-        self.view = champlain.View()
-        self.view.set_property("scroll-mode", champlain.SCROLL_MODE_KINETIC)
+        embed = champlaingtk.ChamplainEmbed()
+        embed.set_size_request(640, 480)
+
+        self.view = embed.get_view()
+        self.view.set_reactive(True)
+        self.view.connect('button-release-event', self.mouse_click_cb, self.view)
+
+        self.view.set_property('scroll-mode', champlain.SCROLL_MODE_KINETIC)
+        self.view.set_property('zoom-level', 5)
+        self.view.set_property('license-text', "Don't eat cereals with orange juice\nIt tastes bad")
+        self.view.set_property('show-scale', True)
 
         self.layer = create_marker_layer(self.view)
         self.view.add_layer(self.layer)
@@ -41,9 +49,6 @@ class LauncherGTK:
         self.view.add_polygon(polygon)
         polygon.hide()
 
-        embed = cluttergtk.Embed()
-        embed.set_size_request(640, 480)
-
         bbox = gtk.HBox(False, 6)
         button = gtk.Button(stock=gtk.STOCK_ZOOM_IN)
         button.connect("clicked", self.zoom_in)
@@ -77,16 +82,14 @@ class LauncherGTK:
         self.spinbutton.set_value(5)
         bbox.add(self.spinbutton)
 
+        button = gtk.Image()
+        self.view.connect("notify::state", self.view_state_changed, button)
+        bbox.pack_end(button, False, False, 0)        
+
         vbox.pack_start(bbox, expand=False, fill=False)
         vbox.add(embed)
 
         self.window.add(vbox)
-        # we need to realize the widget before we get the stage
-        embed.realize()
-
-        stage = embed.get_stage()
-        self.view.set_size(640, 480)
-        stage.add(self.view)
 
         self.window.show_all()
         self.view.center_on(45.466, -73.75)
@@ -105,6 +108,11 @@ class LauncherGTK:
             self.polygon.hide()    
             self.layer.animate_out_all_markers()
 
+    def mouse_click_cb(self, actor, event, view):
+        lat, lon = view.get_coords_from_event(event)
+        print "Mouse click at: %f %f" % (lat, lon)
+        return True
+
     def zoom_changed(self, widget):
         self.view.set_property("zoom-level", self.spinbutton.get_value_as_int())
 
@@ -118,6 +126,13 @@ class LauncherGTK:
     def map_zoom_changed(self, widget, value):
         self.spinbutton.set_value(self.view.get_property("zoom-level"))
 
+    def view_state_changed(self, view, paramspec, image):
+        state = view.get_property("state")
+        if state == champlain.STATE_LOADING:
+            image.set_from_stock(gtk.STOCK_NETWORK, gtk.ICON_SIZE_BUTTON)
+        else:
+            image.clear()
+
 
 if __name__ == "__main__":
     gobject.threads_init()
